/****************************************************************************
** Meta object code from reading C++ file 'kglobalacceld.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.17)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../kglobalaccel-5.116.0/src/runtime/kglobalacceld.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kglobalacceld.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.17. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KGlobalAccelD_t {
    QByteArrayData data[52];
    char stringdata0[795];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KGlobalAccelD_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KGlobalAccelD_t qt_meta_stringdata_KGlobalAccelD = {
    {
QT_MOC_LITERAL(0, 0, 13), // "KGlobalAccelD"
QT_MOC_LITERAL(1, 14, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 30, 20), // "org.kde.KGlobalAccel"
QT_MOC_LITERAL(3, 51, 22), // "yourShortcutGotChanged"
QT_MOC_LITERAL(4, 74, 0), // ""
QT_MOC_LITERAL(5, 75, 8), // "actionId"
QT_MOC_LITERAL(6, 84, 10), // "QList<int>"
QT_MOC_LITERAL(7, 95, 7), // "newKeys"
QT_MOC_LITERAL(8, 103, 20), // "yourShortcutsChanged"
QT_MOC_LITERAL(9, 124, 19), // "QList<QKeySequence>"
QT_MOC_LITERAL(10, 144, 13), // "allComponents"
QT_MOC_LITERAL(11, 158, 22), // "QList<QDBusObjectPath>"
QT_MOC_LITERAL(12, 181, 17), // "allMainComponents"
QT_MOC_LITERAL(13, 199, 18), // "QList<QStringList>"
QT_MOC_LITERAL(14, 218, 22), // "allActionsForComponent"
QT_MOC_LITERAL(15, 241, 6), // "action"
QT_MOC_LITERAL(16, 248, 3), // "key"
QT_MOC_LITERAL(17, 252, 10), // "actionList"
QT_MOC_LITERAL(18, 263, 8), // "shortcut"
QT_MOC_LITERAL(19, 272, 12), // "shortcutKeys"
QT_MOC_LITERAL(20, 285, 15), // "defaultShortcut"
QT_MOC_LITERAL(21, 301, 19), // "defaultShortcutKeys"
QT_MOC_LITERAL(22, 321, 12), // "getComponent"
QT_MOC_LITERAL(23, 334, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(24, 350, 15), // "componentUnique"
QT_MOC_LITERAL(25, 366, 11), // "setShortcut"
QT_MOC_LITERAL(26, 378, 4), // "keys"
QT_MOC_LITERAL(27, 383, 5), // "flags"
QT_MOC_LITERAL(28, 389, 15), // "setShortcutKeys"
QT_MOC_LITERAL(29, 405, 18), // "setForeignShortcut"
QT_MOC_LITERAL(30, 424, 22), // "setForeignShortcutKeys"
QT_MOC_LITERAL(31, 447, 11), // "setInactive"
QT_MOC_LITERAL(32, 459, 10), // "doRegister"
QT_MOC_LITERAL(33, 470, 10), // "unRegister"
QT_MOC_LITERAL(34, 481, 29), // "activateGlobalShortcutContext"
QT_MOC_LITERAL(35, 511, 9), // "component"
QT_MOC_LITERAL(36, 521, 7), // "context"
QT_MOC_LITERAL(37, 529, 23), // "getGlobalShortcutsByKey"
QT_MOC_LITERAL(38, 553, 26), // "QList<KGlobalShortcutInfo>"
QT_MOC_LITERAL(39, 580, 20), // "globalShortcutsByKey"
QT_MOC_LITERAL(40, 601, 23), // "KGlobalAccel::MatchType"
QT_MOC_LITERAL(41, 625, 4), // "type"
QT_MOC_LITERAL(42, 630, 25), // "isGlobalShortcutAvailable"
QT_MOC_LITERAL(43, 656, 23), // "globalShortcutAvailable"
QT_MOC_LITERAL(44, 680, 10), // "unregister"
QT_MOC_LITERAL(45, 691, 14), // "shortcutUnique"
QT_MOC_LITERAL(46, 706, 20), // "blockGlobalShortcuts"
QT_MOC_LITERAL(47, 727, 15), // "SetShortcutFlag"
QT_MOC_LITERAL(48, 743, 10), // "SetPresent"
QT_MOC_LITERAL(49, 754, 13), // "NoAutoloading"
QT_MOC_LITERAL(50, 768, 9), // "IsDefault"
QT_MOC_LITERAL(51, 778, 16) // "SetShortcutFlags"

    },
    "KGlobalAccelD\0D-Bus Interface\0"
    "org.kde.KGlobalAccel\0yourShortcutGotChanged\0"
    "\0actionId\0QList<int>\0newKeys\0"
    "yourShortcutsChanged\0QList<QKeySequence>\0"
    "allComponents\0QList<QDBusObjectPath>\0"
    "allMainComponents\0QList<QStringList>\0"
    "allActionsForComponent\0action\0key\0"
    "actionList\0shortcut\0shortcutKeys\0"
    "defaultShortcut\0defaultShortcutKeys\0"
    "getComponent\0QDBusObjectPath\0"
    "componentUnique\0setShortcut\0keys\0flags\0"
    "setShortcutKeys\0setForeignShortcut\0"
    "setForeignShortcutKeys\0setInactive\0"
    "doRegister\0unRegister\0"
    "activateGlobalShortcutContext\0component\0"
    "context\0getGlobalShortcutsByKey\0"
    "QList<KGlobalShortcutInfo>\0"
    "globalShortcutsByKey\0KGlobalAccel::MatchType\0"
    "type\0isGlobalShortcutAvailable\0"
    "globalShortcutAvailable\0unregister\0"
    "shortcutUnique\0blockGlobalShortcuts\0"
    "SetShortcutFlag\0SetPresent\0NoAutoloading\0"
    "IsDefault\0SetShortcutFlags"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KGlobalAccelD[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      26,   16, // methods
       0,    0, // properties
       2,  246, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    2,  146,    4, 0x46 /* Public | isScriptable */,
       8,    2,  151,    4, 0x46 /* Public | isScriptable */,

 // slots: name, argc, parameters, tag, flags
      10,    0,  156,    4, 0x4a /* Public | isScriptable */,
      12,    0,  157,    4, 0x4a /* Public | isScriptable */,
      14,    1,  158,    4, 0x4a /* Public | isScriptable */,
      15,    1,  161,    4, 0x4a /* Public | isScriptable */,
      17,    1,  164,    4, 0x4a /* Public | isScriptable */,
      18,    1,  167,    4, 0x4a /* Public | isScriptable */,
      19,    1,  170,    4, 0x4a /* Public | isScriptable */,
      20,    1,  173,    4, 0x4a /* Public | isScriptable */,
      21,    1,  176,    4, 0x4a /* Public | isScriptable */,
      22,    1,  179,    4, 0x4a /* Public | isScriptable */,
      25,    3,  182,    4, 0x4a /* Public | isScriptable */,
      28,    3,  189,    4, 0x4a /* Public | isScriptable */,
      29,    2,  196,    4, 0x4a /* Public | isScriptable */,
      30,    2,  201,    4, 0x4a /* Public | isScriptable */,
      31,    1,  206,    4, 0x4a /* Public | isScriptable */,
      32,    1,  209,    4, 0x4a /* Public | isScriptable */,
      33,    1,  212,    4, 0x4a /* Public | isScriptable */,
      34,    2,  215,    4, 0x4a /* Public | isScriptable */,
      37,    1,  220,    4, 0x4a /* Public | isScriptable */,
      39,    2,  223,    4, 0x4a /* Public | isScriptable */,
      42,    2,  228,    4, 0x4a /* Public | isScriptable */,
      43,    2,  233,    4, 0x4a /* Public | isScriptable */,
      44,    2,  238,    4, 0x4a /* Public | isScriptable */,
      46,    1,  243,    4, 0x4a /* Public | isScriptable */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList, 0x80000000 | 6,    5,    7,
    QMetaType::Void, QMetaType::QStringList, 0x80000000 | 9,    5,    7,

 // slots: parameters
    0x80000000 | 11,
    0x80000000 | 13,
    0x80000000 | 13, QMetaType::QStringList,    5,
    QMetaType::QStringList, QMetaType::Int,   16,
    QMetaType::QStringList, QMetaType::QKeySequence,   16,
    0x80000000 | 6, QMetaType::QStringList,    5,
    0x80000000 | 9, QMetaType::QStringList,    5,
    0x80000000 | 6, QMetaType::QStringList,    5,
    0x80000000 | 9, QMetaType::QStringList,    5,
    0x80000000 | 23, QMetaType::QString,   24,
    0x80000000 | 6, QMetaType::QStringList, 0x80000000 | 6, QMetaType::UInt,    5,   26,   27,
    0x80000000 | 9, QMetaType::QStringList, 0x80000000 | 9, QMetaType::UInt,    5,   26,   27,
    QMetaType::Void, QMetaType::QStringList, 0x80000000 | 6,    5,   26,
    QMetaType::Void, QMetaType::QStringList, 0x80000000 | 9,    5,   26,
    QMetaType::Void, QMetaType::QStringList,    5,
    QMetaType::Void, QMetaType::QStringList,    5,
    QMetaType::Void, QMetaType::QStringList,    5,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   35,   36,
    0x80000000 | 38, QMetaType::Int,   16,
    0x80000000 | 38, QMetaType::QKeySequence, 0x80000000 | 40,   16,   41,
    QMetaType::Bool, QMetaType::Int, QMetaType::QString,   16,   35,
    QMetaType::Bool, QMetaType::QKeySequence, QMetaType::QString,   16,   35,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,   24,   45,
    QMetaType::Void, QMetaType::Bool,    4,

 // enums: name, alias, flags, count, data
      47,   47, 0x0,    3,  256,
      51,   47, 0x1,    3,  262,

 // enum data: key, value
      48, uint(KGlobalAccelD::SetPresent),
      49, uint(KGlobalAccelD::NoAutoloading),
      50, uint(KGlobalAccelD::IsDefault),
      48, uint(KGlobalAccelD::SetPresent),
      49, uint(KGlobalAccelD::NoAutoloading),
      50, uint(KGlobalAccelD::IsDefault),

       0        // eod
};

void KGlobalAccelD::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KGlobalAccelD *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->yourShortcutGotChanged((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QList<int>(*)>(_a[2]))); break;
        case 1: _t->yourShortcutsChanged((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QList<QKeySequence>(*)>(_a[2]))); break;
        case 2: { QList<QDBusObjectPath> _r = _t->allComponents();
            if (_a[0]) *reinterpret_cast< QList<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QList<QStringList> _r = _t->allMainComponents();
            if (_a[0]) *reinterpret_cast< QList<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QList<QStringList> _r = _t->allActionsForComponent((*reinterpret_cast< const QStringList(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QStringList _r = _t->action((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 6: { QStringList _r = _t->actionList((*reinterpret_cast< const QKeySequence(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 7: { QList<int> _r = _t->shortcut((*reinterpret_cast< const QStringList(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<int>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QList<QKeySequence> _r = _t->shortcutKeys((*reinterpret_cast< const QStringList(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<QKeySequence>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QList<int> _r = _t->defaultShortcut((*reinterpret_cast< const QStringList(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<int>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QList<QKeySequence> _r = _t->defaultShortcutKeys((*reinterpret_cast< const QStringList(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<QKeySequence>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusObjectPath _r = _t->getComponent((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 12: { QList<int> _r = _t->setShortcut((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QList<int>(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QList<int>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QList<QKeySequence> _r = _t->setShortcutKeys((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QList<QKeySequence>(*)>(_a[2])),(*reinterpret_cast< uint(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QList<QKeySequence>*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->setForeignShortcut((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QList<int>(*)>(_a[2]))); break;
        case 15: _t->setForeignShortcutKeys((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< const QList<QKeySequence>(*)>(_a[2]))); break;
        case 16: _t->setInactive((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 17: _t->doRegister((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 18: _t->unRegister((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 19: _t->activateGlobalShortcutContext((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 20: { QList<KGlobalShortcutInfo> _r = _t->getGlobalShortcutsByKey((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QList<KGlobalShortcutInfo>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QList<KGlobalShortcutInfo> _r = _t->globalShortcutsByKey((*reinterpret_cast< const QKeySequence(*)>(_a[1])),(*reinterpret_cast< KGlobalAccel::MatchType(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QList<KGlobalShortcutInfo>*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->isGlobalShortcutAvailable((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->globalShortcutAvailable((*reinterpret_cast< const QKeySequence(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->unregister((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->blockGlobalShortcuts((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QKeySequence> >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QKeySequence> >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QKeySequence> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (KGlobalAccelD::*)(const QStringList & , const QList<int> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KGlobalAccelD::yourShortcutGotChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (KGlobalAccelD::*)(const QStringList & , const QList<QKeySequence> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&KGlobalAccelD::yourShortcutsChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KGlobalAccelD::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_KGlobalAccelD.data,
    qt_meta_data_KGlobalAccelD,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KGlobalAccelD::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KGlobalAccelD::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KGlobalAccelD.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int KGlobalAccelD::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void KGlobalAccelD::yourShortcutGotChanged(const QStringList & _t1, const QList<int> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KGlobalAccelD::yourShortcutsChanged(const QStringList & _t1, const QList<QKeySequence> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
