/*
    SPDX-FileCopyrightText: 2015 Martin Gräßlin <mgraesslin@kde.org>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#ifndef KGLOBALACCELD_LOGGING_P_H
#define KGLOBALACCELD_LOGGING_P_H
#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(KGLOBALACCELD)

#endif
