/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -i kglobalshortcutinfo_p.h -p kglobalaccel_component_interface /build/kglobalaccel5/src/kglobalaccel-5.116.0/src/org.kde.kglobalaccel.Component.xml
 *
 * qdbusxml2cpp is Copyright (C) 2023 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "kglobalaccel_component_interface.h"

/*
 * Implementation of interface class OrgKdeKglobalaccelComponentInterface
 */

OrgKdeKglobalaccelComponentInterface::OrgKdeKglobalaccelComponentInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgKdeKglobalaccelComponentInterface::~OrgKdeKglobalaccelComponentInterface()
{
}


#include "kglobalaccel_component_interface.moc"
